package com.tzld.piaoquan.api.service.contentplatform.impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.api.common.enums.ExceptionEnum;
import com.tzld.piaoquan.api.common.exception.CommonException;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ContentPlatformGzhPlanMapper;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ContentPlatformGzhPlanVideoMapper;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ContentPlatformQwPlanVideoMapper;
import com.tzld.piaoquan.api.dao.mapper.contentplatform.ext.ContentPlatformPlanMapperExt;
import com.tzld.piaoquan.api.model.config.LoginUserContext;
import com.tzld.piaoquan.api.model.param.contentplatform.*;
import com.tzld.piaoquan.api.model.po.contentplatform.*;
import com.tzld.piaoquan.api.model.vo.GhDetailVo;
import com.tzld.piaoquan.api.model.vo.contentplatform.GzhPlanItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.QwPlanItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.VideoContentItemVO;
import com.tzld.piaoquan.api.remote.AigcApiService;
import com.tzld.piaoquan.api.remote.ManagerApiService;
import com.tzld.piaoquan.api.service.GhDetailService;
import com.tzld.piaoquan.api.service.contentplatform.ContentPlatformCooperateAccountService;
import com.tzld.piaoquan.api.service.contentplatform.ContentPlatformPlanService;
import com.tzld.piaoquan.growth.common.common.enums.GhTypeEnum;
import com.tzld.piaoquan.growth.common.common.enums.StrategyStatusEnum;
import com.tzld.piaoquan.growth.common.model.bo.VideoDetail;
import com.tzld.piaoquan.growth.common.model.po.GhDetail;
import com.tzld.piaoquan.growth.common.service.MessageAttachmentService;
import com.tzld.piaoquan.growth.common.utils.MessageUtil;
import com.tzld.piaoquan.growth.common.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ContentPlatformPlanServiceImpl implements ContentPlatformPlanService {

    @Autowired
    private ContentPlatformGzhPlanMapper gzhPlanMapper;
    @Autowired
    private ContentPlatformPlanMapperExt planMapperExt;
    @Autowired
    private ContentPlatformGzhPlanVideoMapper gzhPlanVideoMapper;
    @Autowired
    private ContentPlatformQwPlanVideoMapper qwPlanVideoMapper;
    @Autowired
    private ContentPlatformCooperateAccountService cooperateAccountService;
    @Autowired
    private GhDetailService ghDetailService;
    @Autowired
    private AigcApiService aigcApiService;
    @Autowired
    private ManagerApiService managerApiService;
    @Autowired
    private MessageAttachmentService messageAttachmentService;

    @Override
    public Page<GzhPlanItemVO> gzhPlanList(GzhPlanListParam param) {
        ContentPlatformAccount loginAccount = LoginUserContext.getUser();
        Page<GzhPlanItemVO> result = new Page<>(param.getPageNum(), param.getPageSize());
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        int count = planMapperExt.getGzhPlanCount(param, loginAccount.getId());
        result.setTotalSize(count);
        if (count == 0) {
            return result;
        }
        List<ContentPlatformGzhPlan> planList = planMapperExt.getGzhPlanList(param,
                loginAccount.getId(), offset, param.getPageSize());
        List<GzhPlanItemVO> list = buildGzhPlanItemVOList(planList);
        result.setObjs(list);
        return result;
    }

    private List<GzhPlanItemVO> buildGzhPlanItemVOList(List<ContentPlatformGzhPlan> planList) {
        if (CollectionUtils.isEmpty(planList)) {
            return null;
        }
        List<Long> accountIds = planList.stream().map(ContentPlatformGzhPlan::getAccountId).collect(Collectors.toList());
        List<ContentPlatformGzhAccount> accountList = cooperateAccountService.getAccountListByIds(accountIds);
        Map<Long, String> accountMap = accountList.stream().collect(Collectors.toMap(ContentPlatformGzhAccount::getId,
                ContentPlatformGzhAccount::getName));
        List<Long> planIds = planList.stream().map(ContentPlatformGzhPlan::getId).collect(Collectors.toList());
        List<ContentPlatformGzhPlanVideo> videoList = getGzhPlanVideoList(planIds);
        Map<Long, List<ContentPlatformGzhPlanVideo>> videoMap = videoList.stream()
                .collect(Collectors.groupingBy(ContentPlatformGzhPlanVideo::getPlanId));
        List<GzhPlanItemVO> result = new ArrayList<>();
        for (ContentPlatformGzhPlan gzhPlan : planList) {
            GzhPlanItemVO planItemVO = new GzhPlanItemVO();
            planItemVO.setId(gzhPlan.getId());
            planItemVO.setAccountId(gzhPlan.getAccountId());
            planItemVO.setAccountName(accountMap.get(gzhPlan.getAccountId()));
            planItemVO.setScene(gzhPlan.getScene());
            planItemVO.setPublishStage(gzhPlan.getPublishStage());
            List<ContentPlatformGzhPlanVideo> videoListByPlanId = videoMap.get(gzhPlan.getId());
            if (CollectionUtils.isNotEmpty(videoListByPlanId)) {
                planItemVO.setVideoCount(videoListByPlanId.size());
                List<String> titleList = videoListByPlanId.stream().map(ContentPlatformGzhPlanVideo::getTitle)
                        .collect(Collectors.toList());
                planItemVO.setTitle(titleList);
            }
            planItemVO.setCreateTimestamp(gzhPlan.getCreateTimestamp());
            result.add(planItemVO);
        }
        return result;
    }

    private List<ContentPlatformGzhPlanVideo> getGzhPlanVideoList(List<Long> planIds) {
        ContentPlatformGzhPlanVideoExample example = new ContentPlatformGzhPlanVideoExample();
        example.createCriteria().andPlanIdIn(planIds);
        return gzhPlanVideoMapper.selectByExample(example);
    }

    @Override
    public void gzhPlanSave(GzhPlanSaveParam param) {
        ContentPlatformAccount loginAccount = LoginUserContext.getUser();
        List<ContentPlatformGzhPlan> planExistList = planMapperExt.getGzhPlanExists(param.getAccountId(), param.getScene(), param.getId());
        if (CollectionUtils.isNotEmpty(planExistList)) {
            throw new CommonException(ExceptionEnum.GZH_SCENE_PLAN_EXISTS);
        }
        if (param.getVideoList().size() > 3) {
            throw new CommonException(ExceptionEnum.GZH_PLAN_VIDEO_SIZE_EXCEED);
        }
        ContentPlatformGzhAccount account = cooperateAccountService.getById(param.getAccountId());
        Long now = System.currentTimeMillis();
        ContentPlatformGzhPlan gzhPlan = new ContentPlatformGzhPlan();
        gzhPlan.setAccountId(param.getAccountId());
        gzhPlan.setScene(param.getScene());
        gzhPlan.setPublishStage(param.getPublishStage());
        gzhPlan.setUpdateTimestamp(now);
        // 更新gh_detail
        List<Long> videoIds = param.getVideoList().stream().map(VideoContentItemVO::getVideoId).collect(Collectors.toList());
        updateGhDetail(account, videoIds);
        // 调用aigc创建发布计划
        String externalId = aigcApiService.createPublishPlan(account.getExternalId(), account.getName());
        if (Objects.isNull(param.getId())) {
            gzhPlan.setExternalId(externalId);
            gzhPlan.setCreateAccountId(loginAccount.getId());
            gzhPlan.setCreateTimestamp(now);
            planMapperExt.insertGzhPlanReturnId(gzhPlan);
        } else {
            gzhPlan.setId(param.getId());
            gzhPlanMapper.updateByPrimaryKeySelective(gzhPlan);
        }
        // 保存视频内容
        saveGzhPlanVideo(param, videoIds, gzhPlan.getId(), loginAccount.getId());
    }

    private void saveGzhPlanVideo(GzhPlanSaveParam param, List<Long> videoIds, Long id, Long loginAccountId) {
        List<ContentPlatformGzhPlanVideo> existsVideo = getGzhVideoByPlanId(id);
        List<Long> existsVideoIds = existsVideo.stream().map(ContentPlatformGzhPlanVideo::getVideoId).collect(Collectors.toList());
        Map<Long, ContentPlatformGzhPlanVideo> existsVideoMap = existsVideo.stream()
                .collect(Collectors.toMap(ContentPlatformGzhPlanVideo::getVideoId, item -> item));
        for (ContentPlatformGzhPlanVideo item : existsVideo) {
            if (!videoIds.contains(item.getVideoId())) {
                gzhPlanVideoMapper.deleteByPrimaryKey(item.getId());
            }
        }
        for (VideoContentItemVO vo : param.getVideoList()) {
            if (existsVideoIds.contains(vo.getVideoId())) {
                ContentPlatformGzhPlanVideo item = existsVideoMap.get(vo.getVideoId());
                item.setTitle(vo.getTitle());
                item.setTitleIsUpdate(vo.getTitleIsEdit());
                item.setCover(vo.getCover());
                item.setCoverIsUpdate(vo.getCoverIsEdit());
                gzhPlanVideoMapper.updateByPrimaryKeySelective(item);
            } else {
                ContentPlatformGzhPlanVideo item = new ContentPlatformGzhPlanVideo();
                item.setPlanId(id);
                item.setVideoId(vo.getVideoId());
                item.setTitle(vo.getTitle());
                item.setTitleIsUpdate(vo.getTitleIsEdit());
                item.setCover(vo.getCover());
                item.setCoverIsUpdate(vo.getCoverIsEdit());
                item.setVideo(vo.getVideo());
                item.setCreateAccountId(loginAccountId);
                item.setCreateTimestamp(System.currentTimeMillis());
                gzhPlanVideoMapper.insertSelective(item);
            }
        }
    }

    private List<ContentPlatformGzhPlanVideo> getGzhVideoByPlanId(Long planId) {
        ContentPlatformGzhPlanVideoExample example = new ContentPlatformGzhPlanVideoExample();
        example.createCriteria().andPlanIdEqualTo(planId);
        return gzhPlanVideoMapper.selectByExample(example);
    }

    private void updateGhDetail(ContentPlatformGzhAccount account, List<Long> videoIds) {
        GhDetail ghDetail = ghDetailService.getGhDetailByGhIdType(account.getGhId(), GhTypeEnum.THIRD_PARTY_GH.type);
        GhDetailVo detailVo = new GhDetailVo();
        if (Objects.nonNull(ghDetail)) {
            ghDetail.setStrategyStatus(StrategyStatusEnum.DEFAULT.status);
            ghDetail.setVideoIds(JSONObject.toJSONString(videoIds));
            ghDetail.setAutoreplySendMinigramNum(videoIds.size());
            ghDetail.setCategory1(account.getContentType());
            BeanUtils.copyProperties(ghDetail, detailVo);
            ghDetailService.updateDetail(detailVo);
        } else {
            ContentPlatformAccount loginUser = LoginUserContext.getUser();
            detailVo.setAccountId(account.getGhId());
            detailVo.setAccountName(account.getName());
            detailVo.setType(GhTypeEnum.THIRD_PARTY_GH.type);
            detailVo.setChannel(loginUser.getChannel());
            detailVo.setCategory1(account.getContentType());
            detailVo.setStrategyStatus(StrategyStatusEnum.DEFAULT.status);
            detailVo.setVideoIds(videoIds);
            detailVo.setAutoreplySendMinigramNum(videoIds.size());
            ghDetailService.addGhDetail(detailVo);
        }
    }

    @Override
    public List<ContentPlatformGzhPlan> getGzhPlanListByCooperateAccountId(Long accountId) {
        ContentPlatformGzhPlanExample example = new ContentPlatformGzhPlanExample();
        example.createCriteria().andAccountIdEqualTo(accountId);
        return gzhPlanMapper.selectByExample(example);
    }

    @Override
    public Page<VideoContentItemVO> getVideoContentList(VideoContentListParam param) {
        Page<VideoContentItemVO> result = new Page<>(param.getPageNum(), param.getPageSize());
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        String dt = planMapperExt.getVideoMaxDt();
        int count = planMapperExt.getVideoCount(param, dt);
        result.setTotalSize(count);
        if (count == 0) {
            return result;
        }
        List<ContentPlatformVideo> videoList = planMapperExt.getVideoList(param, dt, offset, param.getPageSize());
        List<VideoContentItemVO> list = buildVideoContentItemVOList(videoList);
        result.setObjs(list);
        return result;
    }

    private List<VideoContentItemVO> buildVideoContentItemVOList(List<ContentPlatformVideo> videoList) {
        if (CollectionUtils.isEmpty(videoList)) {
            return null;
        }
        Set<Long> videoIds = videoList.stream().map(ContentPlatformVideo::getVideoId).collect(Collectors.toSet());
        Map<Long, VideoDetail> coverMap = messageAttachmentService.getVideoDetail(videoIds);
        List<VideoContentItemVO> result = new ArrayList<>();
        for (ContentPlatformVideo video : videoList) {
            VideoContentItemVO item = new VideoContentItemVO();
            item.setVideoId(video.getVideoId());
            item.setTitle(video.getTitle());
            VideoDetail detail = coverMap.get(video.getVideoId());
            if (Objects.nonNull(detail)) {
                item.setCover(detail.getCover());
            }
            item.setVideo(video.getVideo());
            item.setScore(video.getScore());
            result.add(item);
        }
        return result;
    }

    @Override
    public List<String> getVideoContentCategoryList() {
        String dt = planMapperExt.getVideoMaxDt();
        return planMapperExt.getVideoCategoryList(dt);
    }

    @Override
    public List<String> getVideoContentCoverFrameList(VideoContentCoverFrameParam param) {
        return managerApiService.getCoverImagePaths(param.getVideoId());
    }

    @Override
    public Page<QwPlanItemVO> qwPlanList(QwPlanListParam param) {
        ContentPlatformAccount loginAccount = LoginUserContext.getUser();
        Page<QwPlanItemVO> result = new Page<>(param.getPageNum(), param.getPageSize());
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        int count = planMapperExt.getQwPlanCount(param, loginAccount.getId());
        result.setTotalSize(count);
        if (count == 0) {
            return result;
        }
        List<ContentPlatformQwPlan> planList = planMapperExt.getQwPlanList(param,
                loginAccount.getId(), offset, param.getPageSize());
        List<QwPlanItemVO> list = buildQwPlanItemVOList(planList);
        result.setObjs(list);
        return result;
    }

    private List<QwPlanItemVO> buildQwPlanItemVOList(List<ContentPlatformQwPlan> planList) {
        if (CollectionUtils.isEmpty(planList)) {
            return null;
        }
        List<Long> planIds = planList.stream().map(ContentPlatformQwPlan::getId).collect(Collectors.toList());
        List<ContentPlatformQwPlanVideo> videoList = getQwPlanVideoList(planIds);
        Map<Long, ContentPlatformQwPlanVideo> videoMap = videoList.stream()
                .collect(Collectors.toMap(ContentPlatformQwPlanVideo::getPlanId, Function.identity()));
        List<QwPlanItemVO> result = new ArrayList<>();
        for (ContentPlatformQwPlan qwPlan : planList) {
            QwPlanItemVO planItemVO = new QwPlanItemVO();
            planItemVO.setId(qwPlan.getId());
            planItemVO.setType(qwPlan.getType());
            planItemVO.setScene(qwPlan.getScene());
            planItemVO.setPageUrl(qwPlan.getPageUrl());
            planItemVO.setCreateTimestamp(qwPlan.getCreateTimestamp());
            ContentPlatformQwPlanVideo videoItem = videoMap.get(qwPlan.getId());
            if (Objects.nonNull(videoItem)) {
                planItemVO.setTitle(videoItem.getTitle());
                planItemVO.setCover(videoItem.getCover());
                planItemVO.setVideo(videoItem.getVideo());
            }
            result.add(planItemVO);
        }
        return result;
    }

    private List<ContentPlatformQwPlanVideo> getQwPlanVideoList(List<Long> planIds) {
        ContentPlatformQwPlanVideoExample example = new ContentPlatformQwPlanVideoExample();
        example.createCriteria().andPlanIdIn(planIds);
        return qwPlanVideoMapper.selectByExample(example);
    }

    @Override
    public void qwPlanSave(QwPlanSaveParam param) {
        ContentPlatformAccount loginUser = LoginUserContext.getUser();
        Long now = System.currentTimeMillis();
        for (QwPlanSaveVideoParam videoParam : param.getVideoList()) {
            ContentPlatformQwPlan qwPlan = new ContentPlatformQwPlan();
            qwPlan.setType(param.getType());
            qwPlan.setScene(videoParam.getScene());
            String pageUrl = messageAttachmentService.getPage(loginUser.getChannel(), "wxbdd2a2e93d9a6e25",
                    "dyyjs", "企微", "社群", "位置X", videoParam.getVideoId());
            String rootSourceId = MessageUtil.getRootSourceId(pageUrl);
            qwPlan.setPageUrl(pageUrl);
            qwPlan.setRootSourceId(rootSourceId);
            qwPlan.setCreateAccountId(loginUser.getId());
            qwPlan.setCreateTimestamp(now);
            planMapperExt.insertQwPlanReturnId(qwPlan);
            // 保存视频内容
            saveQwPlanVideo(videoParam, qwPlan.getId(), loginUser.getId());
        }
    }

    private void saveQwPlanVideo(QwPlanSaveVideoParam videoParam, Long planId, Long loginAccountId) {
        ContentPlatformQwPlanVideo item = new ContentPlatformQwPlanVideo();
        item.setPlanId(planId);
        item.setVideoId(videoParam.getVideoId());
        item.setTitle(videoParam.getTitle());
        item.setCover(videoParam.getCover());
        item.setVideo(videoParam.getVideo());
        item.setCreateAccountId(loginAccountId);
        item.setCreateTimestamp(System.currentTimeMillis());
        qwPlanVideoMapper.insertSelective(item);
    }

}
